@@ -16,7 +16,7 @@ class MultiTree:
         self.right = None
         self.level = level
         self.label = label
-    # ǰ,ȡղ㼶ȫڵж [node:level]
+    # 前序遍历,获取按照层级排序的全节点序列对 [node:level]
     def nodes(self,ll={}):
         lll= ll 
         if self.nodeobj is not None:
@@ -32,7 +32,7 @@ class MultiTree:
         for i in lll: 
             llist[i[1]].append(i[0])
         return llist
-    # _õnodeobj:level
+    # 后序遍历_得到nodeobj:level对
     def postorder(self):
         if self.left is not None:
             self.left.postorder()
@@ -40,7 +40,7 @@ class MultiTree:
             self.right.postorder()
         if self.nodeobj is not None:
             print(self.nodeobj,' Level:',self.level,' Label:',self.label)
-    # ǰ_õnodeobj:level
+    # 前序遍历_得到nodeobj:level对
     def preorder(self):
         if self.nodeobj is not None:
             print(self.nodeobj,' Level:',self.level,' Label:',self.label)
@@ -48,44 +48,44 @@ class MultiTree:
             self.left.preorder()
         if self.right is not None:
             self.right.preorder()
-    # 
+    # 层序遍历
     def levelorder(self):
-        # ĳڵ
+        # 返回某个节点的左孩子
         def LChild_Of_Node(node):
             return node.left if node.left is not None else None
-        # ĳڵҺ
+        # 返回某个节点的右孩子
         def RChild_Of_Node(node):
             return node.right if node.right is not None else None
-        # б
+        # 层序遍历列表
         level_order = []
-        # ǷӸڵе
+        # 是否添加根节点中的数据
         if self.nodeobj is not None:
             level_order.append([self])
-        # ĸ߶
+        # 二叉树的高度
         height = self.height()
         if height >= 1:
-            # Եڶ㼰ԺĲв, level_orderӽڵ
+            # 对第二层及其以后的层数进行操作, 在level_order中添加节点而不是数据
             for _ in range(2, height + 1):
-                level = []  # 该层的节?
+                level = []  # 璇ュ眰鐨勮妭鐐?
                 for node in level_order[-1]:
-                    # ӷǿգ
+                    # 如果左孩子非空，则添加左孩子
                     if LChild_Of_Node(node):
                         level.append(LChild_Of_Node(node))
-                    # ҺӷǿգҺ
+                    # 如果右孩子非空，则添加右孩子
                     if RChild_Of_Node(node):
                         level.append(RChild_Of_Node(node))
-                # òǿգӸò
+                # 如果该层非空，则添加该层
                 if level:
                     level_order.append(level)
 
-             # ȡÿе
-            for i in range(0, height):  # 
+             # 取出每层中的数据
+            for i in range(0, height):  # 层数
                 for index in range(len(level_order[i])):
                     level_order[i][index] = level_order[i][index].nodeobj
         return level_order
-    # ĸ߶
+    # 二叉树的高度
     def height(self):
-        # յ߶Ϊ0, ֻrootڵ߶Ϊ1
+        # 空的树高度为0, 只有root节点的树高度为1
         if self.nodeobj is None:
             return 0
         elif self.left is None and self.right is None:
@@ -96,8 +96,8 @@ class MultiTree:
             return 1 + self.left.height()
         else:
             return 1 + max(self.left.height(), self.right.height())
-    # ԭĶҶӽڵ
-    def leaves(self,ll=[],flag = 1): #ζ 0 Ҷӽڵ
+    # 原来的多叉树的叶子节点
+    def leaves(self,ll=[],flag = 1): #意味 0 这个是叶子节点
         if self is None:
             return None
         if flag == 1:
@@ -116,7 +116,7 @@ class MultiTree:
             self.right.leaves(ll,flag = 0)
         return ll
         
-    def leaves_nodeobj(self,ll=[],flag = 1): #ζ 0 Ҷӽڵ
+    def leaves_nodeobj(self,ll=[],flag = 1): #意味 0 这个是叶子节点
         if self is None:
             return None
         if flag == 1:
@@ -135,7 +135,7 @@ class MultiTree:
             self.right.leaves_nodeobj(ll,flag = 0)
         return ll
     
-    def leaves_label(self,ll=[],flag = 1): #ζ 0 Ҷӽڵ
+    def leaves_label(self,ll=[],flag = 1): #意味 0 这个是叶子节点
         if self is None:
             return None
         if flag == 1:
@@ -153,12 +153,12 @@ class MultiTree:
         if self.right is not None:
             self.right.leaves_label(ll,flag = 0)
         return ll
-    #
+    #生成树
     def CreatTree(self):
-        if(self.nodeobj[0] == '('): #ζŻûﵽҶӽ
+        if(self.nodeobj[0] == '('): #存在括号意味着还没达到叶子结点
             node_list = []
-            brackets_num = 0 #Ÿ
-            node_num = 0 #ڵ
+            brackets_num = 0 #括号个数
+            node_num = 0 #节点序号
             node_tmp = ''
             for i in self.nodeobj[1:-1]:
                 if i == '(':
@@ -174,13 +174,13 @@ class MultiTree:
                 else:
                     node_tmp += i
             node_list.append(node_tmp)   
-            #print(node_list)    #鿴ֺĽڵ
+            #print(node_list)    #查看拆分后的节点序列
             self_tmp = self
             for index,item in enumerate(node_list):
                 if index == 0:
-                #һӽڵ㶼Ǹڵڵ
-                #ӽڵһӽڵҽڵ 
-                #label
+                #第一个子节点都是父节点的左节点
+                #后续子节点就是上一个子节点的右节点 
+                #赋予label
                     label = str(index) if self.label == 'root' else self_tmp.label+'_'+str(index)    
                     self.left = MultiTree(item,label=label)
                     self = self.left
@@ -190,17 +190,17 @@ class MultiTree:
                     self.right = MultiTree(item,label=label)
                     self = self.right
                     self.CreatTree()
-    #ҳ㼶d
+    #赋层左子树后序遍历找出最大层级d
     def Posorder_Max_Level(self):
         level = self.level;
         while self.right:
-            #ֵܽ
+            #有右子树就是有兄弟结点
             level = max(level, self.right.level)
             self = self.right
         return level
-    #㼶level
+    #赋予层级level
     def Level(self):
-        if self.left == None: #Ҷӽڵ
+        if self.left == None: #叶子节点
             self.level = 0
         else:
             self.level = self.left.Posorder_Max_Level()+1
@@ -211,7 +211,7 @@ class MultiTree:
             self.right.Postorder_Level()
         if self.nodeobj is not None:
             self.Level()
-    #Ҷӽڵ,Ҫǽڵµڵȷ
+    #叶子节点个数,需要的是节点下的左节点才正确
     def leaf_count(self, flag = 1):
         if self is None:
             return 0
@@ -229,7 +229,7 @@ class MultiTree:
         elif self.right is None and self.left is not None:
             return self.left.leaf_count(flag = 0)
         
-    #ܽڵ
+    #总节点个数
     def node_count(self):
         if self is None:
             return 0
@@ -243,7 +243,7 @@ class MultiTree:
             return 1 + self.left.node_count()
         #else:
             #return self.left.node_count()+ self.right.node_count()
-    #ȡǰڵһӽڵ
+    #获取当前节点一级子节点个数
     def son_count(self):
         self_tmp = self
         son_num = 0
@@ -258,7 +258,7 @@ class MultiTree:
                 self_tmp = self_tmp.right
                 son_num += 1
         return son_num    
-    #ȡǰڵһӽڵself
+    #获取当前节点一级子节点self
     def son(self):
         self_tmp = self
         son_tmp = []
@@ -274,7 +274,7 @@ class MultiTree:
                 self_tmp = self_tmp.right
                 son_tmp.append(self_tmp)
         return son_tmp
-    # õ
+    # 中序遍历，得到生成树数据
     def inorder(self,data=[]):
         if self.left is not None:
             self.left.inorder(data)
@@ -344,7 +344,7 @@ def scoremat(TreeSeqFile:str,
 
     root1.CreatTree()
     root1.Postorder_Level()
-    lll = root1.nodes({}) #άʾ
+    lll = root1.nodes({}) #二维表示
     lllnode = [j for i in lll for j in i]
     lllnode_obj = [j.nodeobj for i in lll for j in i]
     lllnode_label = [j.label for i in lll for j in i]
@@ -353,7 +353,7 @@ def scoremat(TreeSeqFile:str,
 
     oroot1.CreatTree()
     oroot1.Postorder_Level()
-    olll = oroot1.nodes({}) #άʾ
+    olll = oroot1.nodes({}) #二维表示
     olllnode = [j for i in olll for j in i]
     oroot1_label2celltype = leafLable_to_celltype_info(olllnode)
 
@@ -366,7 +366,7 @@ def scoremat(TreeSeqFile:str,
 
     root2.CreatTree()
     root2.Postorder_Level()
-    llll = root2.nodes({}) #άʾ
+    llll = root2.nodes({}) #二维表示
     llllnode = [j for i in llll for j in i]
     llllnode_obj = [j.nodeobj for i in llll for j in i]
     llllnode_label = [j.label for i in llll for j in i]
@@ -375,7 +375,7 @@ def scoremat(TreeSeqFile:str,
 
     oroot2.CreatTree()
     oroot2.Postorder_Level()
-    ollll = oroot2.nodes({}) #άʾ
+    ollll = oroot2.nodes({}) #二维表示
     ollllnode = [j for i in ollll for j in i]
     oroot2_label2celltype = leafLable_to_celltype_info(ollllnode)
 
@@ -410,7 +410,7 @@ def scoremat(TreeSeqFile:str,
         else:
             from tqdm import tqdm
         with tqdm(total=(root1.node_count()*root2.node_count()),desc='Matrix Node') as pbar:
-            #ִѭйɾ(0,0)(0,1)(1,0)(1,1)(0,2)(2,0)(1,2)(2,1)(2,2)(0,3)(3,0)...(n,m)
+            #执行循有规律就是(0,0)(0,1)(1,0)(1,1)(0,2)(2,0)(1,2)(2,1)(2,2)(0,3)(3,0)...(n,m)
             for loop_index in loopindex(root1.level+1,root2.level+1):
                 #print(loop_index)
                 for i in range(lllloop[loop_index[0]]):
@@ -474,25 +474,30 @@ def scoremat(TreeSeqFile:str,
                                                                         llll_label = [i.label for i in llll[0]],
                                                                         merge = merge,)
     
-    def changemat(rac, tracemat, tracemat_value, mat, list_tmp1, list_tmp2):
-            for i in tracemat_value[tuple(rac)]:
-                if isinstance(i[0],int) and isinstance(i[1],int):
-                    list_tmp1.append(tracemat.index[i[0]])
-                    list_tmp2.append(tracemat.columns[i[1]])
-                elif not isinstance(i[0],int) and not isinstance(i[1],int):    
-                    list_tmp1.append(tracemat.index[rac[0]])
-                    list_tmp2.append(tracemat.columns[rac[1]])
-                    return
-                #print(i[0],i[0])
-            for i in tracemat_value[tuple(rac)]:
-                if isinstance(i[0],int) and isinstance(i[1],int):
-                    #print(i[0],root1.leaf_count(), i[1],root2.leaf_count())
-                    if i == [] or (i[0]<root1.leaf_count() and i[1] <root2.leaf_count()):
-                        mat[tuple(i)] = -99999.
-                    else:
-                        mat[tuple(i)] = -99999.
-                        #print(i)
-                        changemat(i, tracemat,tracemat_value, mat, list_tmp1, list_tmp2)
+    def changemat(rac, tracemat, tracemat_value, mat, list_tmp1, list_tmp2, ttused):
+        for i in tracemat_value[tuple(rac)]:
+            if isinstance(i[0],int) and isinstance(i[1],int):
+                list_tmp1.append(tracemat.index[i[0]])
+                #print(tracemat.index[i[0]])
+                list_tmp2.append(tracemat.columns[i[1]])
+                #print(tracemat.index[i[1]])
+            elif not isinstance(i[0],int) and not isinstance(i[1],int):    
+                list_tmp1.append(tracemat.index[rac[0]])
+                #print(tracemat.index[rac[0]])
+                list_tmp2.append(tracemat.columns[rac[1]])
+                return
+                
+            #print(i[0],i[0])
+        for i in tracemat_value[tuple(rac)]:
+            ttused.append(str(i))
+            if isinstance(i[0],int) and isinstance(i[1],int):
+                #print(i[0],root1.leaf_count(), i[1],root2.leaf_count())
+                if i == [] or (i[0]<root1.leaf_count() and i[1] <root2.leaf_count()):
+                    mat[tuple(i)] = -99999.
+                else:
+                    mat[tuple(i)] = -99999.
+                    #print(i)
+                    changemat(i, tracemat,tracemat_value, mat, list_tmp1, list_tmp2, ttused)
             #return mat
     def getmatchtree(rac, lllnode_obj, llllnode_obj, tracemat_value, mat, tree_tmp1, tree_tmp2):
         for i in tracemat_value[tuple(rac)]:
@@ -531,14 +536,15 @@ def scoremat(TreeSeqFile:str,
                 leaves_list_tmp.remove(i)
         return leaves_list_tmp
     
-    if top == 0: #Ĭ
+    if top == 0: #默认情况下
         T1root_T2root = []
         
         mat_tmp = deepcopy(matrix_values)
         list_tmp1 = []
         list_tmp2 = []
         mat_tmp[-1,-1] = -99999.
-        changemat([-1,-1],ttrace, trace_value,mat_tmp, list_tmp1, list_tmp2)
+        ttused = [str([root1.node_count()-1, root2.node_count()-1])]
+        changemat([-1,-1],ttrace, trace_value,mat_tmp, list_tmp1, list_tmp2, ttused)
         
         mat_tmp2 = deepcopy(matrix_values)
         tree_tmp1 = []
@@ -576,7 +582,9 @@ def scoremat(TreeSeqFile:str,
                             'Root1_prune':where_prune(list_tmp1, list(map(lambda x:x.label,root1.leaves([])))),
                             'Root2_prune':where_prune(list_tmp2, list(map(lambda x:x.label,root2.leaves([])))),
                             'row':root1.node_count()-1, 
-                            'col':root2.node_count()-1})
+                            'col':root2.node_count()-1,
+                            'used':ttused
+                             })
 
         return({'matrix':mmatrix, 
                 'tree1_leaves_nodename': oroot1_label2celltype[1],
@@ -603,7 +611,8 @@ def scoremat(TreeSeqFile:str,
             list_tmp1 = []
             list_tmp2 = []
             mat_tmp[del_i_index,del_j_index] = -99999.
-            changemat([del_i_index,del_j_index],ttrace, trace_value,mat_tmp, list_tmp1, list_tmp2)
+            ttused = [str([del_i_index, del_j_index])]
+            changemat([del_i_index,del_j_index],ttrace, trace_value,mat_tmp, list_tmp1, list_tmp2, ttused)
             
             tree_tmp1 = []
             tree_tmp2 = []
@@ -619,21 +628,22 @@ def scoremat(TreeSeqFile:str,
             #    list_tmp1.insert(0,lllnode[del_i_index].label)
             #if list_tmp2[0] != llllnode[del_j_index].label:    
             #    list_tmp2.insert(0,llllnode[del_j_index].label)
-            if jjj > 0:
-                if len(scorelist[-1]['Root1_match']) == 0:
-                    percent = 0
-                else:
-                    percent = (len(list(set(list_tmp1) - set(scorelist[-1]['Root1_match'])) + list(set(scorelist[-1]['Root1_match']) - set(list_tmp1))) / len(scorelist[-1]['Root1_match']))*100.
-                if round(percent) < diff:
-                    #print(round(percent))
+            if jjj > 0 and diff > 0:
+                for i in range(len(scorelist)):
+                    percent = min(percent, len(set(scorelist[i]['used']) - set(ttused)) / len(scorelist[i]['used'])*100.)
+                
+                while percent < float(diff):
                     maxscore = np.max(mat_tmp)
+                    if (maxscore + 99999.) < 0.01:
+                        break
                     del_i_index = np.where(mat_tmp==np.max(mat_tmp))[0][0]
                     del_j_index = np.where(mat_tmp==np.max(mat_tmp))[1][0]
                     
                     list_tmp1 = []
                     list_tmp2 = []
                     mat_tmp[del_i_index,del_j_index] = -99999.
-                    changemat([del_i_index,del_j_index],ttrace, trace_value,mat_tmp, list_tmp1, list_tmp2)
+                    ttused = [str([del_i_index, del_j_index])]
+                    changemat([del_i_index,del_j_index],ttrace, trace_value,mat_tmp, list_tmp1, list_tmp2, ttused)
                     
                     tree_tmp1 = []
                     tree_tmp2 = []
@@ -644,6 +654,9 @@ def scoremat(TreeSeqFile:str,
                     tree_tmp4 = []
                     mat_tmp3[del_i_index,del_j_index] = -99999.
                     getmatchtree([del_i_index,del_j_index],lllnode_label, llllnode_label, trace_value,mat_tmp3, tree_tmp3, tree_tmp4)
+                    percent = 100
+                    for i in range(len(scorelist)):
+                        percent = min(percent, len(set(scorelist[i]['used']) - set(ttused)) / len(scorelist[i]['used'])*100.)
                     
             tree_tmp1.append(';')
             tree_tmp2.append(';')
@@ -668,7 +681,9 @@ def scoremat(TreeSeqFile:str,
                             'Root1_prune':where_prune(list_tmp1, list(map(lambda x:x.label,lllnode[del_i_index].leaves([])))),
                             'Root2_prune':where_prune(list_tmp2, list(map(lambda x:x.label,llllnode[del_j_index].leaves([])))),
                             'row':del_i_index, 
-                            'col':del_j_index})
+                            'col':del_j_index,
+                            'used':ttused
+                             })
 
         return({'matrix':mmatrix, 
                 'tree1_leaves_nodename': oroot1_label2celltype[1],
